package ru.landgrafhomyak.db.skeleton1.api.errors

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.statement.InputParam
import ru.landgrafhomyak.db.skeleton1.api.statement._Statement

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class UninitializedParameterException : RuntimeException, CopyableThrowable<UninitializedParameterException> {
	public val statement: _Statement<*, *>
	public val params: Collection<InputParam<*, *, *>>

	public constructor(statement: _Statement<*, *>, params: Collection<InputParam<*, *, *>>) : super(formatMessage(statement, params)) {
		this.statement = statement
		this.params = params
	}

	public constructor(statement: _Statement<*, *>, params: Collection<InputParam<*, *, *>>, message: String) : super(message) {
		this.statement = statement
		this.params = params
	}

	public constructor(statement: _Statement<*, *>, params: Collection<InputParam<*, *, *>>, message: String, cause: Throwable) : super(message, cause) {
		this.statement = statement
		this.params = params
	}

	public constructor(statement: _Statement<*, *>, params: Collection<InputParam<*, *, *>>, cause: Throwable) : super(formatMessage(statement, params), cause) {
		this.statement = statement
		this.params = params
	}

	public constructor(
		statement: _Statement<*, *>,
		params: Collection<InputParam<*, *, *>>,
		message: String?,
		cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause) {
		this.statement = statement
		this.params = params
	}


	override fun createCopy(): UninitializedParameterException? {
		return UninitializedParameterException(
			statement = this.statement,
			params = this.params,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(statement: _Statement<*, *>, params: Collection<InputParam<*, *, *>>): String =
			"While preparing statement $statement following parameters wasn't initialized: ${params.joinToString()}"
	}
}