package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class NameConflictException : SchemaDefinitionException, CopyableThrowable<NameConflictException> {
	public val scope: Any
	public val nameOwner: Any
	public val name: String

	public constructor(scope: Any, nameOwner: Any, name: String) : super(formatMessage(scope, nameOwner, name)) {
		this.scope = scope
		this.nameOwner = nameOwner
		this.name = name
	}

	public constructor(scope: Any, nameOwner: Any, name: String, customMessage: String) : super(customMessage) {
		this.scope = scope
		this.nameOwner = nameOwner
		this.name = name
	}

	public constructor(scope: Any, nameOwner: Any, name: String, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.scope = scope
		this.nameOwner = nameOwner
		this.name = name
	}


	public constructor(scope: Any, nameOwner: Any, name: String, cause: Throwable) : super(formatMessage(scope, nameOwner, name), cause) {
		this.scope = scope
		this.nameOwner = nameOwner
		this.name = name
	}

	public constructor(
		scope: Any, nameOwner: Any, name: String,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.scope = scope
		this.nameOwner = nameOwner
		this.name = name
	}

	override fun createCopy(): NameConflictException? {
		return NameConflictException(
			scope = this.scope,
			nameOwner = this.nameOwner,
			name = this.name,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			scope: Any, nameOwner: Any, name: String,
		): String = "Name '${name}' already taken by $nameOwner inside $scope"
	}
}