package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException
import ru.landgrafhomyak.db.skeleton1.api.module.Namespace

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class NamespaceAlreadyBusyException : SchemaDefinitionException, CopyableThrowable<NamespaceAlreadyBusyException> {
	public val namespace: Namespace<*, *>

	public constructor(namespace: Namespace<*, *>) : super(formatMessage(namespace)) {
		this.namespace = namespace
	}

	public constructor(namespace: Namespace<*, *>, customMessage: String) : super(customMessage) {
		this.namespace = namespace
	}

	public constructor(namespace: Namespace<*, *>, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.namespace = namespace
	}

	public constructor(namespace: Namespace<*, *>, cause: Throwable) : super(formatMessage(namespace), cause) {
		this.namespace = namespace
	}

	public constructor(
		namespace: Namespace<*, *>,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.namespace = namespace
	}

	override fun createCopy(): NamespaceAlreadyBusyException? {
		return NamespaceAlreadyBusyException(
			namespace = this.namespace,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			namespace: Namespace<*, *>
		): String = "Namespace ${namespace.name} is busy: attempt to allocate something in namespace captured by module or attempt to substitute module to non-empty namespace"
	}
}