package ru.landgrafhomyak.db.skeleton1.api.errors.definition

import kotlin.jvm.JvmStatic
import kotlinx.coroutines.CopyableThrowable
import kotlinx.coroutines.ExperimentalCoroutinesApi
import ru.landgrafhomyak.db.skeleton1.api.DebugApi
import ru.landgrafhomyak.db.skeleton1.api.errors.SchemaDefinitionException

@DebugApi
@OptIn(ExperimentalCoroutinesApi::class)
public class UnreferencedEntityAtUpgradeException : SchemaDefinitionException, CopyableThrowable<UnreferencedEntityAtUpgradeException> {
	public val oldScope: Any
	public val unprocessedEntities: Collection<Any>

	public constructor(oldScope: Any, unprocessedEntities: Collection<Any>) : super(formatMessage(oldScope, unprocessedEntities)) {
		this.oldScope = oldScope
		this.unprocessedEntities = unprocessedEntities
	}

	public constructor(oldScope: Any, unprocessedEntities: Collection<Any>, customMessage: String) : super(customMessage) {
		this.oldScope = oldScope
		this.unprocessedEntities = unprocessedEntities
	}

	public constructor(oldScope: Any, unprocessedEntities: Collection<Any>, customMessage: String, cause: Throwable) : super(customMessage, cause) {
		this.oldScope = oldScope
		this.unprocessedEntities = unprocessedEntities
	}


	public constructor(oldScope: Any, unprocessedEntities: Collection<Any>, cause: Throwable) : super(formatMessage(oldScope, unprocessedEntities), cause) {
		this.oldScope = oldScope
		this.unprocessedEntities = unprocessedEntities
	}

	public constructor(
		oldScope: Any, unprocessedEntities: Collection<Any>,
		message: String?, cause: Throwable?,
		@Suppress("unused", "LocalVariableName") _marker: Unit
	) : super(message, cause, _marker) {
		this.oldScope = oldScope
		this.unprocessedEntities = unprocessedEntities
	}

	override fun createCopy(): UnreferencedEntityAtUpgradeException? {
		return UnreferencedEntityAtUpgradeException(
			oldScope = this.oldScope,
			unprocessedEntities = this.unprocessedEntities,
			message = this.message,
			cause = this.cause,
			_marker = Unit
		)
	}

	public companion object {
		@JvmStatic
		public fun formatMessage(
			scope: Any,
			entities: Collection<Any>,
		): String = "While upgrading $scope following entities wasn't referenced: ${entities.joinToString()}"
	}
}